==> download.php?FileName=terralib-5.2.0.zip: src/terralib/qt/widgets/rp/RasterSlicingWizardPage.cpp <==
/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

//Terralib
#include "../../../dataaccess/utils/Utils.h"
#include "../../../raster/Band.h"
#include "../../../raster/Raster.h"
#include "../../../se/ColorMap.h"
#include "../charts/ChartDisplay.h"
#include "../charts/ChartStyle.h"
#include "../charts/Histogram.h"
#include "../charts/HistogramChart.h"
#include "../charts/HistogramStyle.h"
#include "../charts/Utils.h"
#include "../se/SlicingColorMapWidget.h"
#include "../utils/ScopedCursor.h"
#include "RasterSlicingWizardPage.h"
#include "ui_SlicingColorMapForm.h"
#include "ui_RasterSlicingWizardPageForm.h"

// QT
#include <QComboBox>
#include <QString>

te::qt::widgets::RasterSlicingWizardPage::RasterSlicingWizardPage(QWidget *parent) :
  QWizardPage(parent)
{
  m_ui.reset(new Ui::RasterSlicingWizardPageForm);

  m_ui->setupUi(this);
  m_ui->m_histogramToolButton->setIcon(QIcon::fromTheme("chart-bar"));

  //configure page
  this->setTitle(tr("Raster Slicing"));
  this->setSubTitle(tr("Define the raster slicing parameters."));

  //Creating and adjusting the chart Display's style.
  te::qt::widgets::ChartStyle* chartStyle = new te::qt::widgets::ChartStyle();
  chartStyle->setTitle(tr(""));
  chartStyle->setAxisX(tr(""));
  chartStyle->setAxisY(tr(""));
  chartStyle->setGridChecked(true);

  //Creating and adjusting the chart Display
  QGridLayout* chartLayout = new QGridLayout(m_ui->m_histWidget);

  m_chartDisplay = new te::qt::widgets::ChartDisplay(m_ui->m_histWidget, QString::fromUtf8(""), chartStyle);
  m_chartDisplay->adjustDisplay();
  m_chartDisplay->show();
  m_chartDisplay->replot();

  chartLayout->addWidget(m_chartDisplay, 0, 0);
  chartLayout->setContentsMargins(0, 0, 0, 0);

  m_histogramChart = new te::qt::widgets::HistogramChart(new te::qt::widgets::Histogram());
  m_histogramChart->attach(m_chartDisplay);
  
  m_colorMapWidget = new te::qt::widgets::SlicingColorMapWidget(m_ui->m_slicesWidget);
  m_colorMapWidget->setColorMap(new te::se::ColorMap());

  QGridLayout* slicesWidgetLayout = new QGridLayout(m_ui->m_slicesWidget);

  slicesWidgetLayout->addWidget(m_colorMapWidget);
  slicesWidgetLayout->setContentsMargins(0, 0, 0, 0);

  m_extent.makeInvalid();
  m_srid = 0;
  m_ui->m_visibleAreaCheckBox->setEnabled(false);

  connect(m_ui->m_histogramToolButton, SIGNAL(clicked()), this, SLOT(updateHistogram()));
  connect(m_ui->m_bandComboBox, SIGNAL(currentIndexChanged(int)), m_colorMapWidget, SLOT(onBandSelected(int)));
  connect(m_colorMapWidget->getForm()->m_tableWidget, SIGNAL(cellChanged(int, int)), this, SIGNAL(completeChanged()));
}

te::qt::widgets::RasterSlicingWizardPage::~RasterSlicingWizardPage()
{
}

te::se::ColorMap * te::qt::widgets::RasterSlicingWizardPage::getColorMap()
{
  return m_colorMapWidget->getColorMap();
}

std::string te::qt::widgets::RasterSlicingWizardPage::getCurrentBand()
{
  return m_ui->m_bandComboBox->currentText().toUtf8().data();
}

const te::gm::Envelope& te::qt::widgets::RasterSlicingWizardPage::getExtent()
{
  return m_extent;
}

int te::qt::widgets::RasterSlicingWizardPage::getSRID()
{
  return m_srid;
}

bool te::qt::widgets::RasterSlicingWizardPage::trimRaster()
{
  bool useVisibleArea = m_ui->m_visibleAreaCheckBox->isChecked();
  bool extValid = m_extent.isValid();
  bool isWithinRaster = m_raster->getExtent()->contains(m_extent);

  if (useVisibleArea && extValid && isWithinRaster)
    return true;
  else
    return false;
}

void te::qt::widgets::RasterSlicingWizardPage::setLayer(te::map::AbstractLayerPtr layer)
{
  m_rasterLayer = layer;

  //get input raster
  std::auto_ptr<te::da::DataSet> ds = m_rasterLayer->getData();

  if (ds.get())
  {
    std::size_t rpos = te::da::GetFirstPropertyPos(ds.get(), te::dt::RASTER_TYPE);

    std::auto_ptr<te::rst::Raster> inputRst = ds->getRaster(rpos);

    if (inputRst.get())
    {
      m_raster.reset(inputRst.release());

      m_colorMapWidget->setRaster(m_raster.get());

      size_t bandsNumber = m_raster->getNumberOfBands();

      m_ui->m_bandComboBox->clear();
      for (size_t band = 0; band < bandsNumber; ++band)
      {
        m_ui->m_bandComboBox->addItem(QString::number(band));
      }
    }
  }
}

void te::qt::widgets::RasterSlicingWizardPage::setExtent(const te::gm::Envelope& extent)
{
  m_extent = extent;

  if (m_extent.isValid())
    m_ui->m_visibleAreaCheckBox->setEnabled(true);
  else
    m_ui->m_visibleAreaCheckBox->setEnabled(false);
}

void te::qt::widgets::RasterSlicingWizardPage::setSRID(int srid)
{
  m_srid = srid;
}

void te::qt::widgets::RasterSlicingWizardPage::updateHistogram()
{
  te::qt::widgets::ScopedCursor cursor(Qt::WaitCursor);

  std::unique_ptr<te::da::DataSet> dataset(m_rasterLayer->getData().release());
  std::unique_ptr<te::da::DataSetType> dsType(m_rasterLayer->getSchema().release());

  if (dataset.get())
  {
    std::size_t rpos = te::da::GetFirstPropertyPos(dataset.get(), te::dt::RASTER_TYPE);
    std::auto_ptr<te::rst::Raster> inputRst = dataset->getRaster(rpos);

    if (inputRst.get())
    {
      int band = m_ui->m_bandComboBox->currentIndex();
      int slices = m_ui->m_slicesNumberSpinBox->value();
      bool useVisibleArea = m_ui->m_visibleAreaCheckBox->isChecked();
      bool extValid = m_extent.isValid();
      bool isWithinRaster = inputRst->getExtent()->contains(m_extent);
      std::size_t rpos = te::da::GetFirstPropertyPos(dataset.get(), te::dt::RASTER_TYPE);

      m_histogramChart->setHistogram(te::qt::widgets::createHistogram(dataset.get(), dsType.get(), band, slices, -1));
      m_histogramChart->setData();
    }
  }

  m_chartDisplay->updateLayout();
  m_chartDisplay->replot();
}

void te::qt::widgets::RasterSlicingWizardPage::onSliceBandComboBoxIndexCHanged(int index)
{
  m_ui->m_bandComboBox->setCurrentIndex(index);
}

void te::qt::widgets::RasterSlicingWizardPage::getParameters(
  unsigned int& inputRasterBand, unsigned int& slicesNumber,
  bool& eqHistogram)
{
  inputRasterBand = (unsigned int)m_ui->m_bandComboBox->currentText().toUInt();
  slicesNumber = (unsigned int)m_ui->m_slicesNumberSpinBox->value();
  eqHistogram = m_ui->m_equalizeHistogramCheckBox->isChecked();
}

bool te::qt::widgets::RasterSlicingWizardPage::isComplete() const
{
  if (m_colorMapWidget->getForm()->m_tableWidget->rowCount() > 0)
    return true;
  else
    return false;
}

==> download.php?FileName=terralib-5.3.0.zip: src/terralib/qt/widgets/rp/RasterSlicingWizardPage.cpp <==
/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

This file is part of the TerraLib - a Framework for building GIS enabled applications.

TerraLib is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License,
or (at your option) any later version.

TerraLib is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with TerraLib. See COPYING. If not, write to
TerraLib Team at <terralib-team@terralib.org>.
*/

//Terralib
#include "../../../dataaccess/utils/Utils.h"
#include "../../../raster/Band.h"
#include "../../../raster/Raster.h"
#include "../../../se/ColorMap.h"
#include "../charts/ChartDisplay.h"
#include "../charts/ChartStyle.h"
#include "../charts/Histogram.h"
#include "../charts/HistogramChart.h"
#include "../charts/HistogramStyle.h"
#include "../charts/Utils.h"
#include "../se/SlicingColorMapWidget.h"
#include "../utils/ScopedCursor.h"
#include "RasterSlicingWizardPage.h"
#include "ui_SlicingColorMapForm.h"
#include "ui_RasterSlicingWizardPageForm.h"

// QT
#include <QComboBox>
#include <QString>

te::qt::widgets::RasterSlicingWizardPage::RasterSlicingWizardPage(QWidget *parent) :
  QWizardPage(parent)
{
  m_ui.reset(new Ui::RasterSlicingWizardPageForm);

  m_ui->setupUi(this);
  m_ui->m_histogramToolButton->setIcon(QIcon::fromTheme("chart-bar"));

  //configure page
  this->setTitle(tr("Raster Slicing"));
  this->setSubTitle(tr("Define the raster slicing parameters."));

  //Creating and adjusting the chart Display's style.
  te::qt::widgets::ChartStyle* chartStyle = new te::qt::widgets::ChartStyle();
  chartStyle->setTitle(tr(""));
  chartStyle->setAxisX(tr(""));
  chartStyle->setAxisY(tr(""));
  chartStyle->setGridChecked(true);

  //Creating and adjusting the chart Display
  QGridLayout* chartLayout = new QGridLayout(m_ui->m_histWidget);

  m_chartDisplay = new te::qt::widgets::ChartDisplay(m_ui->m_histWidget, QString::fromUtf8(""), chartStyle);
  m_chartDisplay->adjustDisplay();
  m_chartDisplay->show();
  m_chartDisplay->replot();

  chartLayout->addWidget(m_chartDisplay, 0, 0);
  chartLayout->setContentsMargins(0, 0, 0, 0);

  m_histogramChart = new te::qt::widgets::HistogramChart(new te::qt::widgets::Histogram());
  m_histogramChart->attach(m_chartDisplay);
  
  m_colorMapWidget = new te::qt::widgets::SlicingColorMapWidget(m_ui->m_slicesWidget);
  m_colorMapWidget->setColorMap(new te::se::ColorMap());

  QGridLayout* slicesWidgetLayout = new QGridLayout(m_ui->m_slicesWidget);

  slicesWidgetLayout->addWidget(m_colorMapWidget);
  slicesWidgetLayout->setContentsMargins(0, 0, 0, 0);

  m_extent.makeInvalid();
  m_srid = 0;
  m_ui->m_visibleAreaCheckBox->setEnabled(false);

  connect(m_ui->m_histogramToolButton, SIGNAL(clicked()), this, SLOT(updateHistogram()));
  connect(m_ui->m_bandComboBox, SIGNAL(currentIndexChanged(int)), m_colorMapWidget, SLOT(onBandSelected(int)));
  connect(m_colorMapWidget->getForm()->m_tableWidget, SIGNAL(cellChanged(int, int)), this, SIGNAL(completeChanged()));
}

te::qt::widgets::RasterSlicingWizardPage::~RasterSlicingWizardPage()
{
}

te::se::ColorMap * te::qt::widgets::RasterSlicingWizardPage::getColorMap()
{
  return m_colorMapWidget->getColorMap();
}

std::string te::qt::widgets::RasterSlicingWizardPage::getCurrentBand()
{
  return m_ui->m_bandComboBox->currentText().toUtf8().data();
}

const te::gm::Envelope& te::qt::widgets::RasterSlicingWizardPage::getExtent()
{
  return m_extent;
}

int te::qt::widgets::RasterSlicingWizardPage::getSRID()
{
  return m_srid;
}

bool te::qt::widgets::RasterSlicingWizardPage::trimRaster()
{
  bool useVisibleArea = m_ui->m_visibleAreaCheckBox->isChecked();
  bool extValid = m_extent.isValid();
  bool isWithinRaster = m_raster->getExtent()->contains(m_extent);

  if (useVisibleArea && extValid && isWithinRaster)
    return true;
  else
    return false;
}

void te::qt::widgets::RasterSlicingWizardPage::setLayer(te::map::AbstractLayerPtr layer)
{
  m_rasterLayer = layer;

  //get input raster
  std::unique_ptr<te::da::DataSet> ds = m_rasterLayer->getData();

  if (ds.get())
  {
    std::size_t rpos = te::da::GetFirstPropertyPos(ds.get(), te::dt::RASTER_TYPE);

    std::unique_ptr<te::rst::Raster> inputRst = ds->getRaster(rpos);

    if (inputRst.get())
    {
      m_raster.reset(inputRst.release());

      m_colorMapWidget->setRaster(m_raster.get());

      size_t bandsNumber = m_raster->getNumberOfBands();

      m_ui->m_bandComboBox->clear();
      for (size_t band = 0; band < bandsNumber; ++band)
      {
        m_ui->m_bandComboBox->addItem(QString::number(band));
      }
    }
  }
}

void te::qt::widgets::RasterSlicingWizardPage::setExtent(const te::gm::Envelope& extent)
{
  m_extent = extent;

  if (m_extent.isValid())
    m_ui->m_visibleAreaCheckBox->setEnabled(true);
  else
    m_ui->m_visibleAreaCheckBox->setEnabled(false);
}

void te::qt::widgets::RasterSlicingWizardPage::setSRID(int srid)
{
  m_srid = srid;
}

void te::qt::widgets::RasterSlicingWizardPage::updateHistogram()
{
  te::qt::widgets::ScopedCursor cursor(Qt::WaitCursor);

  std::unique_ptr<te::da::DataSet> dataset(m_rasterLayer->getData().release());
  std::unique_ptr<te::da::DataSetType> dsType(m_rasterLayer->getSchema().release());

  if (dataset.get())
  {
    std::size_t rpos = te::da::GetFirstPropertyPos(dataset.get(), te::dt::RASTER_TYPE);
    std::unique_ptr<te::rst::Raster> inputRst = dataset->getRaster(rpos);

    if (inputRst.get())
    {
      int band = m_ui->m_bandComboBox->currentIndex();

      m_histogramChart->setHistogram(te::qt::widgets::createHistogram(dataset.get(), dsType.get(), band, m_colorMapWidget->getForm()->m_slicesSpinBox->value(), -1));
      m_histogramChart->setData();
    }
  }

  m_chartDisplay->updateLayout();
  m_chartDisplay->replot();
}

void te::qt::widgets::RasterSlicingWizardPage::onSliceBandComboBoxIndexCHanged(int index)
{
  m_ui->m_bandComboBox->setCurrentIndex(index);
}

void te::qt::widgets::RasterSlicingWizardPage::getParameters(
  unsigned int& inputRasterBand, bool& eqHistogram)
{
  inputRasterBand = (unsigned int)m_ui->m_bandComboBox->currentText().toUInt();
  eqHistogram = m_ui->m_equalizeHistogramCheckBox->isChecked();
}

bool te::qt::widgets::RasterSlicingWizardPage::isComplete() const
{
  if (m_colorMapWidget->getForm()->m_tableWidget->rowCount() > 0)
    return true;
  else
    return false;
}
